# A threadsafe (on write), cache for storing objects during the request response cycle.
# User responsibility to provide a serialization | deserialization strategy to set
# get items from the hash.
#
# ex. setting data in Adorn::Cache can be acheieved like so:
# Adorn::Cache.append!(:foo, serialized_object)
#
# ex. retrieving data in Adorn::Cache can be acheieved like so:
# Adorn::Cache[:foo] #=> serialized_object
module Adorn
  module Cache
    class << self
      # Used to create a critical area during writes to Adorn::Cache
      #
      # @return [Mutex]
      def semaphore
        Thread.current[:semaphore] ||= Mutex.new
      end

      # Instantiate or read from Adorn::Cache.
      # Provide an interface to append to Adorn::Cache during request
      #
      # @param [Symbol, *]
      # @return [Hash]
      def append!(key, value)
        Thread.current[:adorn_cache] ||= Hash.new

        semaphore.synchronize do
          Thread.current[:adorn_cache].merge!({ "#{key}" => value })
        end rescue nil #rescue ThreadError
      end

      # Return value at given key from the Adorn::Cache.
      #
      # @param [Symbol, String]
      # @return [Hash]
      def [](key)
        case key.class.to_s
        when 'String'
          Thread.current[:adorn_cache][key]
        when 'Symbol'
          Thread.current[:adorn_cache][key.to_s]
        else
          raise 'Unable to read key'
        end

      end

      # Remove all values from the Adorn::Cache.
      #
      # @return [Hash]
      def clear!
        semaphore.synchronize do
          Thread.current[:adorn_cache].clear
        end rescue nil #rescue ThreadError
      end
    end
  end
end
